pub enum TransformAlgorithm {
EnvelopedSignature,
ExclusiveCanonicalization,
Custom(String),
}Expand description
XML-DSig transform or canonicalization algorithm.
Custom URI values are forwarded to the configured crypto backend and can still fail at runtime when unsupported by that backend.
Variants
EnvelopedSignature
Enveloped-signature transform.
ExclusiveCanonicalization
Exclusive XML canonicalization.
Custom(String)
Backend-specific transform algorithm URI.
Implementations