SpMetadata

Struct SpMetadata 

pub struct SpMetadata { /* private fields */ }
Parsed SP metadata. Derefs to Metadata for the shared accessors.

Implementations

Source

impl SpMetadata

pub fn from_xml(xml: &str) -> Result<Self, SamlError>

Parse SP metadata XML.

Errors

Returns SamlError when XML parsing, parser resource limits, or SP-specific metadata extraction fails.

pub fn from_xml_with_limits( xml: &str, limits: XmlLimits, ) -> Result<Self, SamlError>

Parse SP metadata XML with explicit XML parser resource limits.

Errors

Returns SamlError when XML parsing, parser resource limits, or SP-specific metadata extraction fails.

pub fn is_want_assertions_signed(&self) -> bool

WantAssertionsSigned flag.

pub fn is_authn_request_signed(&self) -> bool

AuthnRequestsSigned flag.

pub fn get_assertion_consumer_service(&self, binding: Binding) -> Option<String>

AssertionConsumerService location for binding.

Methods from Deref<Target = Metadata>

pub fn get_metadata(&self) -> &str

The original metadata XML.

pub fn get_entity_id(&self) -> Option<&str>

entityID.

pub fn get_name_id_format(&self) -> Vec<String>

Declared <NameIDFormat> values.

pub fn x509_certificates(&self, use_: CertUse) -> Vec<String>

All X.509 certificates declared for use (raw, as written in metadata).

pub fn get_x509_certificate(&self, use_: CertUse) -> Option<String>

First X.509 certificate declared for use.

pub fn get_single_logout_service(&self, binding: Binding) -> Option<String>

SingleLogoutService location for binding.

pub fn export_metadata(&self, path: impl AsRef<Path>) -> Result<()>

Write the metadata XML to path.

Errors

Returns std::io::Error if the filesystem write fails.

pub fn get_support_bindings(&self) -> Vec<Binding>

Bindings for which a SingleLogoutService endpoint is declared.

pub fn verify_signature( &self, trusted_certificates: &[String], ) -> Result<bool, SamlError>

Verify this metadata document’s enveloped signature against trusted certificate(s) (federation trust anchor). Requires crypto-bergshamra.

Errors

Returns SamlError when XML parsing, certificate loading, cryptographic verification, or signed <EntityDescriptor> coverage checks fail.

pub fn verify_signature_with_limits( &self, trusted_certificates: &[String], limits: XmlLimits, ) -> Result<bool, SamlError>

Verify this metadata document’s signature with explicit XML parser limits.

Errors

Returns SamlError when XML parsing, certificate loading, cryptographic verification, or signed <EntityDescriptor> coverage checks fail.

pub fn verify_signature_detailed( &self, trusted_certificates: &[String], ) -> Result<MetadataSignatureVerification, SamlError>

Verify this metadata document’s signature and preserve signed <EntityDescriptor> coverage evidence using default XML parser limits.

Errors

Returns SamlError when XML parsing, certificate loading, cryptographic verification, transform policy, or signed <EntityDescriptor> coverage checks fail.

pub fn verify_signature_detailed_with_limits( &self, trusted_certificates: &[String], limits: XmlLimits, ) -> Result<MetadataSignatureVerification, SamlError>

Verify this metadata document’s signature and preserve signed <EntityDescriptor> coverage evidence.

Errors

Returns SamlError when XML parsing, certificate loading, cryptographic verification, transform policy, or signed <EntityDescriptor> coverage checks fail.